package com.utopiaindustries.hseobservationsapp.activities.ObservationForms;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.lifecycle.ViewModelProvider;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.activities.DashboardActivity;
import com.utopiaindustries.hseobservationsapp.utils.HSERequestModel;
import com.utopiaindustries.hseobservationsapp.utils.StorageManager.StorageManager;
import com.utopiaindustries.hseobservationsapp.utils.ProgressDialogFragment;
import com.utopiaindustries.hseobservationsapp.viewmodels.LoginViewModel;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;


public class ObservationThreeActivity extends AppCompatActivity {

    RadioGroup rg1;
    ImageView imgBack;
    Button btnSubmit, btnDraft;
    EditText etDescription;

    LoginViewModel loginViewModel;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_observation_three);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        initializeLayouts();

        imgBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        etDescription.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                StorageManager.getInstance().getObservationsModel().get(0).setDescription(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        rg1.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener()
        {
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (checkedId == R.id.rb_open) {
                    StorageManager.getInstance().getObservationsModel().get(0).setObservationStatus(true);
                } else if (checkedId == R.id.rb_closed) {
                    StorageManager.getInstance().getObservationsModel().get(0).setObservationStatus(false);
                }
            }
        });

        btnSubmit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                alertReportSubmit(ObservationThreeActivity.this);
                /*Toast.makeText(ObservationThreeActivity.this,"Reported Submitted",Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(ObservationThreeActivity.this, DashboardActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);*/
            }
        });

        btnDraft.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });
    }

    private void initializeLayouts() {

        rg1 = findViewById(R.id.rg1);
        imgBack = findViewById(R.id.img_back);
        etDescription = findViewById(R.id.et_description);
        etDescription.setImeOptions(EditorInfo.IME_ACTION_DONE);
        etDescription.setRawInputType(InputType.TYPE_CLASS_TEXT);

        btnSubmit = findViewById(R.id.btn_submit);
        btnDraft = findViewById(R.id.btn_draft);

        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        loginViewModel.getLoadingState().observe(this, isLoading -> {

            if (isLoading != null && isLoading) {
                showProgressDialog();
            } else {
                dismissProgressDialog();
            }
        });

        loginViewModel.getErrorMessage().observe(this, errorResponse -> {
            Toast.makeText(this, errorResponse, Toast.LENGTH_SHORT).show();
        });

        loginViewModel.getUserSaveLiveData().observe(this, hseSaveResponse -> {
            if (hseSaveResponse != null && hseSaveResponse.getStatus().equals("success")) {
                Toast.makeText(this, "Reported Submitted", Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(ObservationThreeActivity.this, DashboardActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
            }
        });
    }

    public void alertReportSubmit(Context con) {
        ViewGroup viewGroup = findViewById(android.R.id.content);

        TextView dialogOkBtn, dialogCancelBtn;

        AlertDialog.Builder builder = new AlertDialog.Builder(con);
        View view1 = LayoutInflater.from(con).inflate(R.layout.custom_layout_for_report_submission, viewGroup, false);
        builder.setCancelable(false);
        builder.setView(view1);

        dialogOkBtn = view1.findViewById(R.id.dialogOkBtn);
        dialogCancelBtn = view1.findViewById(R.id.dialogCancelBtn);

        AlertDialog alertDialog = builder.create();
        alertDialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));

        dialogOkBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();

               // byte[] reportBytes = convertReportToByteArray();
                /*HSERequestModel requestModel = new HSERequestModel();
                requestModel.setSite_id(StorageManager.getInstance().getSite_id());
                requestModel.setSupervisorId(StorageManager.getInstance().getSupervisorId());
                requestModel.setObservation_date(StorageManager.getInstance().getObservation_date());
                requestModel.setShift(StorageManager.getInstance().getShift());
                requestModel.setRecordTypeId(StorageManager.getInstance().getRecordTypeId());
                requestModel.setObservationModel(StorageManager.getInstance().getObservationsModel());*/

                Gson gson = new GsonBuilder().setPrettyPrinting().create();
                String jsonRequest = gson.toJson(StorageManager.getInstance());
                Log.e("RequestModel JSON", jsonRequest);

                //loginViewModel.saveHSEData(requestModel);


            }
        });

        dialogCancelBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                alertDialog.dismiss();
                //Toast.makeText(con, "Cancel", Toast.LENGTH_SHORT).show();

            }
        });

        alertDialog.show();
    }

    private byte[] convertReportToByteArray() {
        byte[] bytes = new byte[0];
        ByteArrayOutputStream byteArrayOutputStream = null;
        ObjectOutputStream objectOutputStream = null;
        try {
            StorageManager observationStorageManager = StorageManager.getInstance();
            // convert into byte array
            byteArrayOutputStream = new ByteArrayOutputStream();
            objectOutputStream = new ObjectOutputStream( byteArrayOutputStream );
            objectOutputStream.writeObject( observationStorageManager );
            objectOutputStream.flush();
            bytes =  byteArrayOutputStream.toByteArray();

        } catch (  Exception  e ) {
            e.printStackTrace();
        }
        finally {
            try {
                if (objectOutputStream != null) objectOutputStream.close();
                if (byteArrayOutputStream != null) byteArrayOutputStream.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }

        return bytes;
    }

    public void showProgressDialog() {
        ProgressDialogFragment progressDialog = new ProgressDialogFragment();
        progressDialog.setCancelable(false);
        progressDialog.show(getSupportFragmentManager(), "progressDialog");
    }

    public void dismissProgressDialog() {
        ProgressDialogFragment progressDialog = (ProgressDialogFragment)
                getSupportFragmentManager().findFragmentByTag("progressDialog");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
    }
}